from __future__ import division
__author__ = 'yoelk'

import wx
from wx import xrc
import copy
import sys
import  wx.lib.filebrowsebutton as filebrowse
import wx.lib.masked as  masked
from instrumentino import cfg
import pickle
from executable_listctrl import ExecutableListCtrl
from method import SysMethod

class MethodsListCtrl(ExecutableListCtrl):
    '''
    A list of methods to be run
    '''
    def __init__(self, parent, methods=[]):
        ExecutableListCtrl.__init__(self, parent, xrc.XRCCTRL(parent, 'methodsListPanel'), {1: 'Method', 2: 'Repeat'}, '=== run sequence ===', methods) 

    def getDefaultDataItem(self):
        '''
        Return an empty method item
        '''
        return SysMethod()
     
    def getFirstColumnWidget(self, panel, listDataItem):
        '''
        Return the widget for the first column (a file dialog in this case)
        '''
        fileBrowser = filebrowse.FileBrowseButton(panel,
                                                  labelText='',
                                                  dialogTitle='Choose a method file',
                                                  startDirectory=cfg.UserFilesPath(),
                                                  fileMask=cfg.methodWildcard,
                                                  fileMode=wx.OPEN,
                                                  changeCallback=listDataItem.onPathChanged)
        filename = listDataItem.methodFileName
        if not listDataItem.checkSystemCompatibility(filename):
            filename = ''        
        fileBrowser.SetValue(filename)
        return fileBrowser
     
    def setOtherColumns(self, index, listDataItem):
        '''
        Set the other columns' content
        '''
        item = self.list.GetItem(index, self.columnNameToNum['Repeat'])
        item.SetWindow(listDataItem.setRepeatPanel(self.list), expand=True)
        self.list.SetItem(item)        


from __future__ import division
__author__ = 'yoelk'
from datetime import datetime
import wx
from wx import xrc
from instrumentino.comp import SysVarAnalog, SysVarDigital
from instrumentino import cfg

import matplotlib
from matplotlib.dates import DateFormatter, MinuteLocator, SecondLocator,\
    AutoDateFormatter, AutoDateLocator
matplotlib.use('WXAgg')
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import \
    FigureCanvasWxAgg as FigCanvas, \
    NavigationToolbar2WxAgg as NavigationToolbar
from matplotlib import pyplot as plt
import pylab as pl
from mpl_toolkits.axes_grid1 import host_subplot
import mpl_toolkits.axisartist as AA

class LogGraphPanel(wx.Panel):
    '''
    A panel with a log graph
    adapted from examples by Eli Bendersky (eliben@gmail.com)
    '''
    def __init__(self, parent, sysComps):
        '''
        Creates the main panel with all the controls on it
        '''
        wx.Panel.__init__(self, parent)
        self.parent = parent
        self.sysComps = sysComps
        self.dataWriteBulk = 10

        # add controls
        self.cb_freeze = wx.CheckBox(self, -1, "Freeze")
        self.Bind(wx.EVT_CHECKBOX, self.on_cb_freeze, self.cb_freeze)
        self.slider_label = wx.StaticText(self, -1, "Zoom (min): ")
        self.slider_zoom = wx.Slider(self, -1, value=1, minValue=1, maxValue=60, style=wx.SL_AUTOTICKS | wx.SL_LABELS)
        self.slider_zoom.SetTickFreq(10, 1)
        self.Bind(wx.EVT_COMMAND_SCROLL_THUMBTRACK, self.on_slider_width, self.slider_zoom)

        # set main plot
        self.dpi = 100
        self.axisSeparation = 45
        self.mainAxes = host_subplot(111, axes_class=AA.Axes)
        pl.gca().axes.get_yaxis().set_ticks([])
        self.mainAxes.xaxis_date()
        self.canvas = FigCanvas(self, -1, pl.gcf())
        self.toolbar = NavigationToolbar(self.canvas)
                
        # set individual data lines
        self.lastTime = None
        self.time = []
        self.analogData = {}
        self.analogPlotData = {}
        self.yRange = {}
        self.digitalData = {}
        self.allData = {}
        self.analogAxes = {}
        plotNum = 0
        for comp in self.sysComps:
            for var in comp.vars.values():
                name = var.FullName()
                if isinstance(var, SysVarAnalog):
                    self.analogData[name] = []
                    self.yRange[name] = var.range
                    self.analogAxes[name] = self.mainAxes.twinx()
                    self.analogAxes[name].axis["right"] = self.analogAxes[name].get_grid_helper().new_fixed_axis(loc="right",
                                                                                                                 axes=self.analogAxes[name],
                                                                                                                 offset=(plotNum*self.axisSeparation, 0))
                    plotNum += 1
                    
                if isinstance(var, SysVarDigital):
                    self.digitalData[name] = []
                
        self.allData.update(self.analogData)
        self.allData.update(self.digitalData)
        
        # modify the main plot
        plt.subplots_adjust(right=1-(0.085*len(self.analogData)))
        self.mainAxes.set_axis_bgcolor('white')
        pl.setp(self.mainAxes.get_xticklabels(), fontsize=8)
        pl.setp(self.mainAxes.get_yticklabels(), fontsize=8)
        
        # Show on screen
        self.hbox = wx.BoxSizer(wx.HORIZONTAL)
        self.hbox.Add(self.cb_freeze, 0)
        self.hbox.AddSpacer(30)
        self.hbox.Add(self.slider_label, 0)
        self.hbox.Add(self.slider_zoom, 1)
        self.hbox.Add(self.toolbar, 0, wx.EXPAND | wx.ALIGN_LEFT)
                
        self.vbox = wx.BoxSizer(wx.VERTICAL)
        self.vbox.Add(self.canvas, 1, wx.EXPAND | wx.GROW | wx.ALL)
        self.vbox.Add(self.hbox, 0, wx.EXPAND)
                
        # fit all to screen
        self.parent.GetSizer().Add(self, 1, wx.EXPAND | wx.GROW | wx.ALL)
        self.SetSizer(self.vbox)
        self.vbox.Fit(self)
        
    def AddData(self, name, value):
        # keep all data arrays the same length. the time array should be updated last
        if len(self.allData[name]) <= len(self.time):
            if value == None:
                value = self.allData[name][-1] if len(self.allData[name]) > 0 else 0
                
            self.allData[name] += [value]
            
    def FinishUpdate(self):
        self.time += [datetime.now()]

        # write a header with variable names
        if len(self.time) == 1:
            cfg.signalsLogFile.write('time,' + str(self.allData.keys())[1:-1] + '\r')
            
        # update the signals' file once in a while
        if len(self.time) % self.dataWriteBulk == 0:
            for idx in range(-1*self.dataWriteBulk,0):
                cfg.signalsLogFile.write(str(self.time[idx].strftime('%H:%M:%S.%f')) + ',' + str([v[idx] for v in self.allData.values()])[1:-1] + '\r')
                        
        # only show the graph when there's at least 2 data points
        if len(self.time) < 2:
            return
        self.Redraw(len(self.time) == 2)

    def StopUpdates(self):
        for idx in range(-1*(len(self.time) % self.dataWriteBulk),0):
            cfg.signalsLogFile.write(str(self.time[idx].strftime('%H:%M:%S.%f')) + ',' + str([v[idx] for v in self.allData.values()])[1:-1] + '\r')
            
        # close extra plotting window
        pl.close()
    
    def Redraw(self, firstTime=False):
        """ Redraws the figure
        """
        if firstTime:
            self.mainAxes.get_xaxis().set_major_formatter(DateFormatter('%H:%M'))
            self.mainAxes.get_xaxis().set_major_locator(MinuteLocator())
        
        for name in self.analogData.keys():
            if firstTime:
                self.analogPlotData[name] = self.analogAxes[name].plot(self.time, self.analogData[name], linewidth=1)[0]
                self.analogAxes[name].axis["right"].label.set_color(self.analogPlotData[name].get_color())
                self.analogAxes[name].axis["right"].toggle(all=True)
                self.analogAxes[name].set_ylabel(name)
            
            if not self.cb_freeze.IsChecked():
                self.analogAxes[name].set_xbound(lower=self.time[max(0,len(self.time)-int(self.slider_zoom.GetValue() * 60 * cfg.app.updateFrequency))],
                                                 upper=self.time[-1])
                self.analogAxes[name].set_ybound(lower=self.yRange[name][0],
                                                 upper=self.yRange[name][1])
                
            self.analogPlotData[name].set_xdata(self.time)
            self.analogPlotData[name].set_ydata(self.analogData[name])
        
        pl.setp(self.mainAxes.get_xticklabels(), visible=True)
        
        self.canvas.draw()
    
    def on_cb_freeze(self, event):
        self.Redraw()
    
    def on_slider_width(self, event):
        self.Redraw()

##############################
class SimpleFrame(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, None)
        self.SetSizer(wx.BoxSizer(wx.HORIZONTAL))
        self.panel = LogGraphPanel(self, [])

if __name__ == '__main__':
    app = wx.PySimpleApp()
    app.frame = SimpleFrame()
    app.frame.Show()
    app.MainLoop()



from __future__ import division
__author__ = 'yoelk'

import wx
from wx import xrc, lib
from wx.lib import wordwrap
import pickle
import os
from datetime import datetime
from instrumentino import cfg
from method import ActionsListCtrl
from sequence import MethodsListCtrl
from log_graph import LogGraphPanel
from instrumentino.action import SysAction, SysActionParamTime
from instrumentino.controllers.arduino import Arduino
from util import SerialUtil
import time
from pkg_resources import resource_filename
import sys

class InstrumentinoApp(wx.App):
    '''
    This class implements the application
    '''
    monitorUpdateDelayMilisec = Arduino.cacheReadDelayMilisec
    updateFrequency = 1000 / monitorUpdateDelayMilisec

    def __init__(self, system):
        self.system = system
        self.sysComps = self.system.comps
        self.sysActions = self.system.actions
        wx.App.__init__(self, False)

    def OnInit(self):
        '''
        Load the main window from the main.xrc
        '''
        self.mainXrc = wx.xrc.XmlResource(cfg.ResourcePath('main.xrc'))
        self.InitFrame()
        return True

    def InitFrame(self):
        '''
        Init the main window
        '''
        self.listButtons = []
        self.runButtons = []
        self.mainFrame = self.mainXrc.LoadFrame(None, 'mainFrame')
        self.splitter = wx.xrc.XRCCTRL(self.mainFrame, 'splitter')
        self.mainFrame.SetTitle(self.system.name)
        self.mainFrame.Bind(wx.EVT_CLOSE, self.OnClose)
        
        # log
        self.logGraph = LogGraphPanel(wx.xrc.XRCCTRL(self.mainFrame, 'logGraphPage'), self.sysComps)
        self.Connect(-1, -1, cfg.EVT_LOG_UPDATE, self.OnLogUpdate)
        
        # update framework
        cfg.InitVariables(self)
        
        # Menu
        self.mainFrame.Bind(wx.EVT_MENU, self.OnLoadSequence, id=wx.xrc.XRCID('loadSequenceMenuItem'))
        self.mainFrame.Bind(wx.EVT_MENU, self.OnSaveSequence, id=wx.xrc.XRCID('saveSequenceMenuItem'))
        self.mainFrame.Bind(wx.EVT_MENU, self.OnLoadMethod, id=wx.xrc.XRCID('loadMethodMenuItem'))
        self.mainFrame.Bind(wx.EVT_MENU, self.OnSaveMethod, id=wx.xrc.XRCID('saveMethodMenuItem'))
        self.mainFrame.Bind(wx.EVT_MENU, self.OnClose, id=wx.xrc.XRCID('quitMenuItem'))
        self.mainFrame.Bind(wx.EVT_MENU, self.OnAbout, id=wx.xrc.XRCID('aboutMenuItem'))
        
        menusDict = dict(self.mainFrame.GetMenuBar().GetMenus())
        commMenu = [key for key, value in menusDict.iteritems() if value == 'Comm'][0]
        for comp in self.sysComps:
            cfg.AddControllerIfNeeded(comp.controllerClass)
        
        for controller in cfg.controllers:
            menu = commMenu.Append(-1, controller.name, 'Connect the ' + controller.name)
            self.mainFrame.Bind(wx.EVT_MENU, controller.OnMenuConnect, menu)            
        
        # sysCompsPanel
        monitorPanel = wx.xrc.XRCCTRL(self.mainFrame, 'monitorPanel')
        self.stopButton = wx.BitmapButton(monitorPanel, -1, wx.Bitmap(cfg.ResourcePath('stopButton.png'), wx.BITMAP_TYPE_PNG))
        monitorPanel.GetSizer().Add(self.stopButton, flag=wx.BOTTOM|wx.ALIGN_CENTRE_HORIZONTAL)
        self.stopButton.Bind(wx.EVT_BUTTON, self.OnStopButton)               
                
        self.sysCompsPanel = wx.xrc.XRCCTRL(self.mainFrame, 'sysCompsPanel')
        sysCompsPanelBoxSizer = self.sysCompsPanel.GetSizer()        
        for sysComp in self.sysComps:
            panel = sysComp.CreatePanel(self.sysCompsPanel)
            if panel != None:
                sysCompsPanelBoxSizer.Add(panel)
        
        # make all sysComps fill their given area
        for sysCompPanel in sysCompsPanelBoxSizer.GetChildren():
            sysCompPanel.SetFlag(wx.GROW)
        sysCompsPanelBoxSizer.Fit(self.mainFrame)
 
        self.Connect(-1, -1, cfg.EVT_UPDATE_CONTROLS, self.OnUpdateControls)
        
        # Method
        self.actionsListCtrl = ActionsListCtrl(wx.xrc.XRCCTRL(self.mainFrame, 'methodPage'), self.sysActions)
        self.runButtons.append(self.actionsListCtrl.runButton)
        self.listButtons.extend([self.actionsListCtrl.list, self.actionsListCtrl.addButton, self.actionsListCtrl.removeButton])
        
        # Sequence
        self.methodsListCtrl = MethodsListCtrl(wx.xrc.XRCCTRL(self.mainFrame, 'sequencePage'))
        self.runButtons.append(self.methodsListCtrl.runButton)
        self.listButtons.extend([self.methodsListCtrl.list, self.methodsListCtrl.addButton, self.methodsListCtrl.removeButton])

        # This makes sure both pages are drawn
        notebook = wx.xrc.XRCCTRL(self.mainFrame, 'methodsAndSequences')
        notebook.SendPageChangedEvent(0,1)        
        self.mainFrame.GetSizer().Fit(self.mainFrame)        
        notebook.SendPageChangedEvent(1,0)
        
        # main frame
        self.splitter.SetSashPosition(400, True)
        self.mainFrame.GetSizer().Fit(self.mainFrame)
        self.UpdateControls()
        self.mainFrame.Show()
        
        # Monitor periodically
        self.timer = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, self.MonitorUpdate, self.timer)
        self.timer.Start(self.monitorUpdateDelayMilisec)
        
    def OnLogUpdate(self, event):
        '''
        Update log
        '''
        (text, critical) = event.data
        cfg.Log(text)
        if critical:
            dlg = wx.MessageDialog(self.mainFrame,
                               text,
                               'Error', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
    
    def OnUpdateControls(self, event):
        '''
        Update the system components' availability
        '''
        self.UpdateControls(event.data)
    
    def UpdateControls(self, runningOperation=False):
        '''
        Update the system components frame with actual values
        '''
        for comp in self.sysComps:
            comp.Enable(cfg.IsCompOnline(comp))
        
        for button in self.listButtons:
            button.Enable(not runningOperation)

        for button in self.runButtons:
            button.Enable(cfg.AllOnline() and not runningOperation)

        self.stopButton.Enable(cfg.AllOnline() and runningOperation)
        
        cfg.logTextCtrl.Enable(cfg.AllOnline())
        cfg.logGraph.Enable(cfg.AllOnline())
    
    def OnLoadSequence(self, event):
        '''
        Load a sequence file
        '''
        self.loadFile(event, "Choose a sequence file", cfg.sequenceWildcard, self.methodsListCtrl)
    
    def OnSaveSequence(self, event):
        '''
        Save a sequence file
        '''
        self.saveFile(event, "Save sequence as ...", cfg.sequenceWildcard, '.seq', self.methodsListCtrl)
    
    def OnLoadMethod(self, event):
        '''
        Load a method file
        '''
        self.loadFile(event, "Choose a method file", cfg.methodWildcard, self.actionsListCtrl)

    def OnSaveMethod(self, event):
        '''
        Save a method file
        '''
        self.saveFile(event, "Save method as ...", cfg.methodWildcard, '.mtd', self.actionsListCtrl)
            
    def loadFile(self, event, message, wildcard, listCtrl):
        '''
        File loading helper function
        '''
        dlg = wx.FileDialog(self.mainFrame, message=message, defaultDir=cfg.UserFilesPath(),
                            defaultFile="", wildcard=wildcard,
                            style=wx.OPEN | wx.CHANGE_DIR)

        if dlg.ShowModal() == wx.ID_OK:
            try:
                filePath = dlg.GetPaths()[0]
                with open(filePath, 'r') as fp:
                    dlg.Destroy()
                    savedFile = pickle.load(fp)
                    if savedFile.systemUid == self.system.GetSystemUid():
                        listCtrl.populateList(savedFile.list)
                        cfg.LogFromOtherThread('File loaded: ' + filePath)
                    else:
                        errDlg = wx.MessageDialog(self.mainFrame, 'File incompatible with this system',
                                              'Error',
                                              wx.OK | wx.ICON_ERROR)
                        errDlg.ShowModal()
                        errDlg.Destroy()
                    
            except (IOError, EOFError):
                errDlg = wx.MessageDialog(self.mainFrame, 'Error loading the file',
                                          'Error',
                                          wx.OK | wx.ICON_ERROR)
                errDlg.ShowModal()
                errDlg.Destroy()
    
    def saveFile(self, event, message, wildcard, extension, listCtrl):
        '''
        File saving helper function
        '''
        dlg = wx.FileDialog(self.mainFrame, message=message, defaultDir=cfg.UserFilesPath(),
                            defaultFile="", wildcard=wildcard, style=wx.SAVE)

        if dlg.ShowModal() == wx.ID_OK:
            try:
                path = dlg.GetPath()
                name = os.path.splitext(path)[0]
                with open(name + extension, 'w') as fp:
                    pickle.dump(SavedFile(self.system.GetSystemUid(), listCtrl.getDataItemsList()), fp)
                    cfg.LogFromOtherThread('File saved: ' + path)
                    fp.close()
            except (IOError, EOFError):
                errDlg = wx.MessageDialog(self.mainFrame, 'Error saving the file',
                               'Error',
                               wx.OK | wx.ICON_ERROR)
                errDlg.ShowModal()
                errDlg.Destroy()
        
        dlg.Destroy()
    
    def OnClose(self, event):
        '''
        Close the application
        '''
        dlg = wx.MessageDialog(self.mainFrame,
                               "Do you really want to Close this application?",
                               "Confirm Exit", wx.OK|wx.CANCEL|wx.ICON_QUESTION)
        result = dlg.ShowModal()
        dlg.Destroy()
        if result == wx.ID_OK:
            cfg.userStopped = True
            self.timer.Stop()
            
            if cfg.commandsLogFile != None:
                cfg.commandsLogFile.close()
            
            cfg.logGraph.StopUpdates()
            if cfg.signalsLogFile != None:
                cfg.signalsLogFile.close()
                
            cfg.Close()

            self.mainFrame.Destroy()

    def OnComm(self, controllerName, connectionSetupFunc):
        '''
        Communication setup helper function
        '''
        dlg = wx.SingleChoiceDialog(
                self.mainFrame, 'Select the serial port', 'Connect ' + controllerName,
                SerialUtil().getSerialPortsList(),
                wx.CHOICEDLG_STYLE
                )

        if dlg.ShowModal() == wx.ID_OK:
            connectionSetupFunc(dlg.GetStringSelection())

        dlg.Destroy()
                
    def OnStopButton(self, evt):
        '''
        Respond to stop button being pressed
        '''
        cfg.userStopped = True
        dlg = wx.MessageDialog(self.mainFrame, 'Operation stopped!',
                               'Notification',
                               wx.OK | wx.ICON_INFORMATION
                               )
        result = dlg.ShowModal()
        dlg.Destroy()
        
    def OnAbout(self, evt):
        '''
        Show about dialog
        '''
        # First we create and fill the info object
        info = wx.AboutDialogInfo()
        info.Name = self.system.name
        info.Version = self.system.version
        info.Copyright = "(C) 2013 University of Basel"
        info.Description = wx.lib.wordwrap.wordwrap(self.system.description + '\r\nIt is based on the instrumentino package.', 
                                    350, wx.ClientDC(self.mainFrame))
        info.WebSite = ("http://www.chemie.unibas.ch/~hauser/instrumentino.html")
        info.Developers = [ "Joel Koenka" ]

        info.License = wx.lib.wordwrap.wordwrap(
            'This software is released under GPLv3.'
            "When using instrumentino for published scientific use, you're requested to cite the original release article"
            'For more information, please contact: yoelk@tx.technion.ac.il',
            500, wx.ClientDC(self.mainFrame))

        # Then we call wx.AboutBox giving it that info object
        wx.AboutBox(info)

    def MonitorUpdate(self, event):
        '''
        Read system variables' values from controllers
        '''
        for comp in self.sysComps:
            if cfg.IsCompOnline(comp):
                comp.Update()
        
        if cfg.AllOnline():
            self.logGraph.FinishUpdate()
            

class SavedFile(object):
    '''
    Describe a system dependent saved file
    '''
    def __init__(self, systemUid, list):
        self.systemUid = systemUid
        self.list = list


class Instrument():
    '''
    an instrument parent class
    '''
    def __init__(self, comps, actions, version='1.0', name='Instrument', description='Instrument\'s description'):
        self.comps = comps
        self.actions = actions
        self.version = version
        self.name = name
        self.description = description
        
        self.StartApp()

    def GetSystemUid(self):
        '''
        Return a unique id for the instrument
        '''
        return self.name + self.version + self.description
    
    def StartApp(self):
        '''
        Run application
        '''
        app = InstrumentinoApp(self)
        app.MainLoop()
        
#################################
if __name__ == '__main__':
    Instrument((), (SysAction('an action', (SysActionParamTime(),)),), '1', 'test', 'testing')


from __future__ import division
__author__ = 'yoelk'

from instrumentino import cfg
import wx
import time
import datetime
from executable_listctrl import RunnableItem

class SysActionParam(object):
    '''
    An action's parameter
    
    Sub-classes should maintain of self.maskedTextCtrl and implement:
    SetTextCtrl(parent) - Set the text controls for the parameter's value
    Get()               - Get the value from the text control
    '''
    def __init__(self, name, units='', value=''):
        self.name = name
        self.units = units
        self.value = value
        self.maskedTextCtrl = None
        
    def onKeyPress(self, event):
        '''
        Update the parameter's value when changed in panel
        '''
        self.value = self.maskedTextCtrl.GetValue()
        event.Skip()
        
    def __getstate__(self):
        '''
        Copy the object's state from self.__dict__ which contains
        all our instance attributes. Always use the dict.copy()
        method to avoid modifying the original state.
        '''
        state = self.__dict__.copy()
        # Remove the unpicklable entries.
        if 'maskedTextCtrl' in state:
            del state['maskedTextCtrl']
        return state


class SysActionParamInt(SysActionParam):
    '''
    An action's integer parameter
    '''
    def __init__(self, name, theRange, units='', value=0):
        SysActionParam.__init__(self, name, units, str(value))
        self.range = theRange
        
    def SetTextCtrl(self, parent):
        '''
        Set the text controls for the parameter's value
        '''
        self.maskedTextCtrl = wx.lib.masked.NumCtrl(parent, -1, self.value, 
                                             integerWidth=cfg.numIntegerPartWidth,
                                             fractionWidth=0,
                                             min=self.range[0],
                                             max=self.range[1])
        self.maskedTextCtrl.Bind(wx.EVT_KEY_UP, self.onKeyPress)
        return self.maskedTextCtrl
        
    def Get(self):
        '''
        Get the value from the text control
        '''
        return int(self.value)


class SysActionParamFloat(SysActionParam):
    '''
    An action's float parameter
    '''
    def __init__(self, sysVarAnalog=None, range=[0,0], name='', units='', value=0):
        if sysVarAnalog == None:            
            theName = name
            theRange = range
            theUnits = units
        else:
            theName = name if name != '' else sysVarAnalog.name
            theRange = range if range != [0,0] else sysVarAnalog.range
            theUnits = units if units != '' else sysVarAnalog.units
        
        SysActionParam.__init__(self, theName, theUnits, str(value))
        self.range = theRange
        
    def SetTextCtrl(self, parent):
        '''
        Set the text controls for the parameter's value
        '''
        self.maskedTextCtrl = wx.lib.masked.NumCtrl(parent, -1, self.value,
                                             integerWidth=cfg.numIntegerPartWidth,
                                             fractionWidth=cfg.numFractionPartWidth,
                                             min=self.range[0],
                                             max=self.range[1])
        self.maskedTextCtrl.Bind(wx.EVT_KEY_UP, self.onKeyPress)
        return self.maskedTextCtrl
        
    def Get(self):
        '''
        Get the value from the text control
        '''
        return float(self.value)


class SysActionParamTime(SysActionParam):
    '''
    An action's time parameter
    '''
    def __init__(self, name='Time', value='00:00:00.000'):
        SysActionParam.__init__(self, name, 'h:m:s.ms', value)
        
    def SetTextCtrl(self, parent):
        '''
        Set the text controls for the parameter's value
        '''
        self.maskedTextCtrl = wx.lib.masked.TextCtrl(parent, -1, '',
                                                     mask="##:##:##.###",
                                                     formatcodes='DF!',
                                                     defaultValue=str(self.value))
        self.maskedTextCtrl.Bind(wx.EVT_KEY_UP, self.onKeyPress)
        return self.maskedTextCtrl
        
    def Get(self):
        '''
        Get the value from the text control
        '''
        timeValue = time.strptime(self.value.split('.')[0], '%H:%M:%S')
        hours = timeValue.tm_hour
        minutes = timeValue.tm_min
        seconds = timeValue.tm_sec if len(self.value.split('.')) < 2 else timeValue.tm_sec + int(self.value.split('.')[1]) / 1000
        return datetime.timedelta(hours=hours, minutes=minutes, seconds=seconds).total_seconds()


class SysAction(RunnableItem):
    '''
    A runnale action
    
    Sub-classes should implement:
    Command() - Run the command
    '''
    def __init__(self, name, params=()):
        self.name = name
        self.params = params
        self.paramsPanel = None
        self.commandFinished = False
        
    def __repr__(self):
        '''
        Return a printed representation for the action
        '''
        retVal = 'sysAction:\r' + 'name: ' + self.name + '\rparams:'
        for param in self.params:
            retVal += ' ' + param.name + '=' + str(param.value)
        return retVal
    
    def SetParamsPanel(self, parent):
        '''
        Set the parameters' panel
        '''
        paramsPanel = wx.Panel(parent)
        sizer = wx.GridSizer(0,2)
        for param in self.params:
            unitsStr = ' (' + param.units + ')' if param.units != '' else ''
            sizer.Add(wx.StaticText(paramsPanel, -1, param.name + unitsStr + ': '))
            sizer.Add(param.SetTextCtrl(paramsPanel), flag=wx.EXPAND)
        paramsPanel.SetSizer(sizer)
        
        return paramsPanel
    
    def Run(self, Log=True):
        '''
        Run the action
        '''
        if Log:
            logMsg = datetime.datetime.now().strftime('%X ')
            logMsg += self.name
            for idx, param in enumerate(self.params):
                if idx == 0:
                    logMsg += ': '
                else:
                    logMsg += ', '
                logMsg += param.name + '=' + str(param.value) 
            cfg.LogFromOtherThread(logMsg)

        # Call the sub-class' command function
        self.Command()


from __future__ import division
from instrumentino.util import SerialUtil
__author__ = 'yoelk'

from instrumentino import cfg
import serial
import wx

class InstrumentinoController(object):
    '''
    base class for instrumentino compatible controllers
    '''
    
    def __init__(self, name):
        '''
        init
        '''
        self.name = name
        self.online = False
        
    def __str__(self):
        return self.name + " is on port %s" %(self.serial.port)

    def OnMenuConnect(self, event):
        '''
        Select a comm port to connect to
        '''
        dlg = wx.SingleChoiceDialog(
                cfg.mainFrame, 'Select the serial port', 'Connect ' + self.name,
                SerialUtil().getSerialPortsList(),
                wx.CHOICEDLG_STYLE
                )

        if dlg.ShowModal() == wx.ID_OK:
            self.online = self.Connect(dlg.GetStringSelection())
            cfg.UpdateControlsFromOtherThread()
            return self.online

        dlg.Destroy()

    def Connect(self, port):
        '''
        Connect to the controller. To be implemented by subclass.
        Returns - True (for succesful connection) or False
        '''
        pass
    
    def Close(self):
        '''
        Close the connection to the controller. To be implemented by subclass.
        '''
        pass


from __future__ import division
__author__ = 'yoelk'

from ctypes import *
from instrumentino.controllers import InstrumentinoController
from instrumentino.comp import SysVarDigital, SysComp
from instrumentino import cfg
import time
from instrumentino.util import Chdir
from threading import Semaphore

class LabSmithEIB(InstrumentinoController):
    '''
    This class implements an interface to a LabSmith EIB controller 
    '''

    '''
    EIB constants
    '''
    MIN_DEVICE_ADDRESS = 0x01
    MAX_DEVICE_ADDRESS = 0x7F
    
    DEVICE_TYPE_SPS01 = 1
    DEVICE_TYPE_4VM01 = 2
    
    VALVE_STATE_UNKNOWN = 0
    VALVE_STATE_UNCHANGED = 0
    VALVE_STATE_A = 1
    VALVE_STATE_CLOSED = 2
    VALVE_STATE_B = 3
    
    SYRINGE_PUMP_MAX_POWER = 0xA0

    valveStateToValue = {'A': VALVE_STATE_A,
                         'B': VALVE_STATE_B,
                         'closed': VALVE_STATE_CLOSED,
                         'unchanged': VALVE_STATE_UNCHANGED}
    valveValueToState = {v: k for k, v in valveStateToValue.items()}
    
    name = 'LabSmith EIB'
    
    def __init__(self):
        InstrumentinoController.__init__(self, self.name)
        self.accessSemaphore = Semaphore()
        self.syringePump = None

    def Connect(self, port):
        '''
        port - the name of the serial port. This works on unix and windows systems, given the appropriate name
        '''
        # This works only in windows, so port numbers are COM1,COM2 etc.
        portNumber = int(port.replace('COM', ''))
        
        tempDirChange = Chdir(cfg.ResourcePath())
#         self.DLL = CDLL('uProcessDriver_C_V1_2_1.dll')
        self.DLL = CDLL('uProcessDriver_C.dll')
        del tempDirChange
        self.EIB = self.DLL.NewEIB()
        if self.EIB == 0:
            return False
        retCode = self.DLL.ConnectEIB(self.EIB, portNumber)
        if retCode != 0:
            return False
        self.syringePump = None
        self.valves = None
        
        DeviceDataArray = c_ubyte * (self.MAX_DEVICE_ADDRESS+1)
        deviceAddresses = DeviceDataArray()
        deviceTypes = DeviceDataArray()
        devicesNum = self.DLL.ScanDevices(self.EIB, byref(deviceAddresses), byref(deviceTypes))
        if not devicesNum > 0:
            return False
        for idx in range(devicesNum):
            if deviceTypes[idx] == self.DEVICE_TYPE_SPS01:
                self.syringePump = self.DLL.NewSPS01(self.EIB, c_ubyte(deviceAddresses[idx]))
                retCode = self.DLL.InitSyringe(self.syringePump)
                if retCode == 0:
                    return False
            if deviceTypes[idx] == self.DEVICE_TYPE_4VM01:
                self.valves = self.DLL.New4VM01(self.EIB, c_ubyte(deviceAddresses[idx]))
                retCode = self.DLL.InitValves(self.valves)
                if retCode == 0:
                    return False
        
        return True
    
    def Close(self):
        if self.syringePump != None:
            self.StopSyringe()
            
    def SetValves(self, **kwargs):
        self.accessSemaphore.acquire(True)
        valves = ['unchanged', 'unchanged', 'unchanged', 'unchanged']
        for name, value in kwargs.items():
            valveNum = int(name.replace('valve', ''))
            valves[valveNum-1] = value
        
        self.DLL.SetValves(self.valves,
                           c_int(self.valveStateToValue[valves[0]]),
                           c_int(self.valveStateToValue[valves[1]]),
                           c_int(self.valveStateToValue[valves[2]]),
                           c_int(self.valveStateToValue[valves[3]]))
        time.sleep(0.7)
        self.accessSemaphore.release()
        
    def GetValves(self):
        self.accessSemaphore.acquire(True)
        valves = [c_int(), c_int(), c_int(), c_int()]
        self.DLL.GetValves(self.valves,
                           byref(valves[0]),
                           byref(valves[1]),
                           byref(valves[2]),
                           byref(valves[3]))
        for idx, _ in enumerate(valves):
            try:
                valves[idx] = self.valveValueToState[valves[idx].value]
            except:
                valves[idx] = 0
        
        self.accessSemaphore.release()
        return valves
    
    def SetSyringePower(self, percent):
        self.accessSemaphore.acquire(True)
        self.DLL.SetSyringePower(self.syringePump, c_int(int(percent / 100 * self.SYRINGE_PUMP_MAX_POWER)))
        self.accessSemaphore.release()
        
    def SetSyringeSpeed(self, percent):
        self.accessSemaphore.acquire(True)
        self.DLL.SetSyringeSpeed(self.syringePump, c_int(int(percent / 100 * self.DLL.GetSyringeMaxSpeed(self.syringePump))))
        self.accessSemaphore.release()
        
    def SetSyringeDiameterAndGetMaxVolume(self, diameterMiliMeter):
        self.accessSemaphore.acquire(True)
        self.DLL.SetSyringeDiameter(self.syringePump, c_double(diameterMiliMeter))
        maxVolume = self.DLL.GetSyringeMaxVolume(self.syringePump)
        self.accessSemaphore.release()
        return maxVolume

    def MoveSyringeToPosition(self, pos):
        self.accessSemaphore.acquire(True) 
        self.DLL.MoveSyringeToPosition(self.syringePump, c_int(pos))
        time.sleep(6)
        self.accessSemaphore.release()
        
    def MoveSyringeToVolumePercent(self, percent, maxVolume):
        self.accessSemaphore.acquire(True)
        self.DLL.MoveSyringeToVolume(self.syringePump, c_double(percent / 100 * maxVolume))
        time.sleep(6)
        self.accessSemaphore.release()
        
    def StopSyringe(self):
        self.accessSemaphore.acquire(True)
        self.DLL.StopSyringe(self.syringePump)
        self.accessSemaphore.release()


# base class and variables        
class SysCompLabSmith(SysComp):
    '''
    A LabSmith component base class
    '''
    def __init__(self, name, vars, helpLine=''):
        SysComp.__init__(self, name, vars, LabSmithEIB, helpLine)

class SysVarDigitalLabSmith_AV201Position(SysVarDigital):
    '''
    A LabSmith AV201 valve position
    '''
    states = ('A', 'closed', 'B')
    
    def __init__(self, name, valvesControllerPort, helpLine='', editable=True):
        SysVarDigital.__init__(self, name, self.states, LabSmithEIB, helpLine=helpLine, editable=editable)
        self.valvesController = None
        self.valvesControllerPort = valvesControllerPort

    def SetController(self, valvesController):
        self.valvesController = valvesController
        self.compName = valvesController.name
        
    def GetFunc(self):
        state = self.valvesController.getValve(self.valvesControllerPort) 
        return state if state in self.states else None
    
    def SetFunc(self, state):
        self.valvesController.setValve(self.valvesControllerPort, state)


from __future__ import division
from instrumentino.controllers.arduino import SysCompArduino
from collections import OrderedDict
__author__ = 'yoelk'

from instrumentino import cfg

class TecanSia(SysCompArduino):
    # currently only N0/N1 is supported
    strokeToSeconds = OrderedDict({'N1': 
                                   {0: 1.25,
                                    1: 1.30,
                                    2: 1.39,
                                    3: 1.52,
                                    4: 1.71,
                                    5: 1.97,
                                    6: 2.37,
                                    7: 2.77,
                                    8: 3.03,
                                    9: 3.36,
                                    10: 3.77,
                                    11: 4.30,
                                    12: 5.00,
                                    13: 6.00,
                                    14: 7.50,
                                    15: 10.00,
                                    16: 15.00,
                                    17: 30.00,
                                    18: 31.58,
                                    19: 33.33,
                                    20: 35.29,
                                    21: 37.50,
                                    22: 40.00,
                                    23: 42.86,
                                    24: 46.15,
                                    25: 50.00,
                                    26: 54.55,
                                    27: 60.00,
                                    28: 66.67,
                                    29: 75.00,
                                    30: 85.71,
                                    31: 100.00,
                                    32: 120.00,
                                    33: 150.00,
                                    34: 200.00,
                                    35: 300.00,
                                    36: 333.33,
                                    37: 375.00,
                                    38: 428.57,
                                    39: 500.00, 
                                    40: 600.00}, 
                                   'N2': {}})
    
    DtCmdStart = '/'
    DtCmdEnd = '\r'
    DtCmdExecute = 'R'
    
    serialBaudrate = 9600
    pumpMaxMicroSteps = 48000
    
    def __init__(self, name, pumpVolumeMiliLit, addressPump, addressMultivalve, pinRx=None, pinTx=None, serialPort=1):
        SysCompArduino.__init__(self, 'SIA', (), "send commands to the SIA")
        self.serialPort = serialPort
        if (pinRx != None and pinTx != None):
            self.pinRx = pinRx
            self.pinTx = pinTx
            self.useSoftSer = True
        else:
            self.useSoftSer = False
        self.pumpVolumeMiliLit = pumpVolumeMiliLit
        self.addressPump = addressPump
        self.addressMultivalve = addressMultivalve
        
    def sendCommand(self, address, command, waitForAnswerSec=None):
        self.GetController().SerSend(self.DtCmdStart + address + command + self.DtCmdExecute + self.DtCmdEnd, waitForAnswerSec, self.useSoftSer, self.serialPort)

    def miliLitToMicroSteps(self, miliLit):
        return str(int(miliLit * self.pumpMaxMicroSteps / self.pumpVolumeMiliLit))

    def InitPumpAndMultivalve(self, pumpInitCmd='N1ZJ0', valveInitCmd='ZJ0'):
        self.sendCommand(self.addressPump, pumpInitCmd, waitForAnswerSec=3)
        self.sendCommand(self.addressMultivalve, valveInitCmd, waitForAnswerSec=1)
        
    def selectMultivalvePort(self, port, moveDirection='O', waitForAnswerSec=1):
        self.sendCommand(self.addressMultivalve, moveDirection + str(port), waitForAnswerSec)
    
    def _pullOrDispenseAtMultivalvePort(self, pullOrDispense, port, miliLit, speed, moveDirection, waitForAnswerSec):
        self.selectMultivalvePort(port, moveDirection, waitForAnswerSec)
        volumeFraction = miliLit / self.pumpVolumeMiliLit
        speedIndex = self.speedToSecondsPerStrokeIndex(speed)
        self.sendCommand(self.addressPump, 'OS' + str(speedIndex) + 'M1000' + pullOrDispense + self.miliLitToMicroSteps(miliLit), self.strokeToSeconds['N1'][speedIndex] * volumeFraction + 2)
        
    def _pullOrDispenseAtPumpInputPort(self, pullOrDispense, miliLit, speed):
        volumeFraction = miliLit / self.pumpVolumeMiliLit
        speedIndex = self.speedToSecondsPerStrokeIndex(speed)
        self.sendCommand(self.addressPump, 'IS' + str(speedIndex) + 'M1000' + pullOrDispense + self.miliLitToMicroSteps(miliLit), self.strokeToSeconds['N1'][speedIndex] * volumeFraction + 2)
        
    def pullFromMultivalvePort(self, port, miliLit, speed, moveDirection='I', waitForAnswerSec=1):
        self._pullOrDispenseAtMultivalvePort('P', port, miliLit, speed, moveDirection, waitForAnswerSec)
        
    def dispenseToMultivalvePort(self, port, miliLit, speed, moveDirection='I', waitForAnswerSec=1):
        self._pullOrDispenseAtMultivalvePort('D', port, miliLit, speed, moveDirection, waitForAnswerSec)
        
    def pullFromPumpInputPort(self, miliLit, speed):
        self._pullOrDispenseAtPumpInputPort('P', miliLit, speed)
        
    def dispenseToPumpInputPort(self, miliLit, speed):
        self._pullOrDispenseAtPumpInputPort('D', miliLit, speed)
        
    def TransferFromInputToMultivalvePort(self, port, miliLit, speed, moveDirection='I', waitForAnswerSec=1):
        self.selectMultivalvePort(port, moveDirection, waitForAnswerSec)
        volumeFraction = (miliLit*2) / self.pumpVolumeMiliLit
        speedIndex = self.speedToSecondsPerStrokeIndex(speed)
        self.sendCommand(self.addressPump, 'IS' + str(speedIndex) + 'M1000P' +self.miliLitToMicroSteps(miliLit) + 
                             'OD' + self.miliLitToMicroSteps(miliLit), self.strokeToSeconds['N1'][speedIndex] * volumeFraction + 2)    
        
    def FirstTimeOnline(self):
        if self.useSoftSer:
            self.GetController().SoftSerConnect(self.pinRx, self.pinTx, self.serialBaudrate, self.serialPort)
        else:
            self.GetController().HardSerConnect(self.serialBaudrate, self.serialPort)
        return super(TecanSia, self).FirstTimeOnline()
    
    def speedToSecondsPerStrokeIndex(self, microLitPerSec, microstepMode = 'N1'):
        miliLitPerSec = microLitPerSec / 1000
        secPerStroke = 1 / (miliLitPerSec / self.pumpVolumeMiliLit)
        for indexString, val in self.strokeToSeconds[microstepMode].items():
            if secPerStroke < val:
                return indexString
        # if reached here, return the slowest possibility
        return self.strokeToSeconds[microstepMode].keys()[-1]


from __future__ import division
from instrumentino.controllers.labsmith_eib import SysCompLabSmith
__author__ = 'yoelk'

from instrumentino import cfg
import wx
import time
from instrumentino.comp import SysVarDigital, SysComp


"""
system components
"""

class LabSmithValves4VM01(SysCompLabSmith):
    def __init__(self, name, digiVars):
        SysCompLabSmith.__init__(self, name, digiVars, 'open/close valves')
        
        for var in digiVars:
            var.SetController(self)
            
        self.valvesDriver = None
        
    def setValve(self, port, state):
        if port == 1:
            self.GetController().SetValves(valve1=state)
        if port == 2:
            self.GetController().SetValves(valve2=state)
        if port == 3:
            self.GetController().SetValves(valve3=state)
        if port == 4:
            self.GetController().SetValves(valve4=state)
            
    def getValve(self, port):
        valves = self.GetController().GetValves()
        return valves[port-1]


class LabSmithSPS01SyringePump(SysCompLabSmith):
    '''
    A LabSmith SPS01 syringe pump
    '''
    syringeVolumeMicroLitToDiameterMiliMeter = {100: 3.256, # -80 (100 ul): .1282" (3.256 mm)
                                                50: 2.304,  # -40 (50 ul): .0907" (2.304 mm)
                                                20: 1.458,  # -20 (20 ul): .0574" (1.458 mm)
                                                10: 1.031,  # -08 (10 ul): .0406" (1.031 mm) 
                                                5: 0.729}   # -04 (5 ul): .0287" (0.729 mm) 
    
    def __init__(self, name, volumeMicroLit, helpline='', defaultSpeedPercent = 50, defaultPowerPercent = 75):
        SysCompLabSmith.__init__(self, name, (), 'move syringe pump')
        self.defaultSpeedPercent = defaultSpeedPercent
        self.defaultPowerPercent = defaultPowerPercent
        self.diameterMiliMeter = self.syringeVolumeMicroLitToDiameterMiliMeter[volumeMicroLit]
        self.maxVolume = None
        
    def FirstTimeOnline(self):
        self.maxVolume = self.GetController().SetSyringeDiameterAndGetMaxVolume(self.diameterMiliMeter)
        self.GetController().SetSyringeSpeed(self.defaultSpeedPercent)
        self.GetController().SetSyringePower(self.defaultPowerPercent)
        self.setVolumePercent(5)
    
    def setVolumePercent(self, volumePercent):
        self.GetController().MoveSyringeToVolumePercent(volumePercent, self.maxVolume)



from __future__ import division
from instrumentino.controllers.arduino import SysCompArduino, SysVarDigitalArduino,\
    SysVarAnalogArduinoUnipolar, SysVarAnalogArduinoBipolarWithExternalPolarity
__author__ = 'yoelk'

from instrumentino import cfg

class Spellman(SysCompArduino):
    ''' Supports either a unipolar voltage controller or a bipolar voltage controller with a symmetric range [-X,X] '''
    def __init__(self, name, rangeV, rangeI, pinEnable, pinInV, pinInI, pinOutV, pinOutI=None, highFreqPWM=False, pinsVoltsMax=5, safetyMaxAbsVoltage=None):
        varEnable = SysVarDigitalArduino('enable', pinEnable, name)
        
        # Reduce maximal pin output according to security voltage
        if safetyMaxAbsVoltage != None:
            divider = safetyMaxAbsVoltage / max(abs(rangeV[0]), abs(rangeV[1]))
        else:
            divider = 1
        rangeV[0] *= divider
        rangeV[1] *= divider
        pinsVoltsMaxAdjusted = pinsVoltsMax * divider
        
        # set voltage variable according to the voltage range. Use methods defined in the instantiated sub-class 
        if rangeV[0] * rangeV[1] >= 0:
            voltageVar = SysVarAnalogArduinoUnipolar('V', rangeV, pinInV, pinOutV, name, 'Voltage', 'kV', self.PreEditV, highFreqPWM=highFreqPWM, pinOutVoltsMax=pinsVoltsMaxAdjusted, pinInVoltsMax=pinsVoltsMaxAdjusted)
        else:
            voltageVar = SysVarAnalogArduinoBipolarWithExternalPolarity('V', rangeV, pinInV, pinOutV, self.SetPolarityPositive, self.GetPolarityPositive, name, 'Voltage', 'kV', self.PreEditV, highFreqPWM=highFreqPWM, pinOutVoltsMax=pinsVoltsMaxAdjusted, pinInVoltsMax=pinsVoltsMaxAdjusted)            
        
        currentVar = SysVarAnalogArduinoUnipolar('I', rangeI, pinInI, pinOutI, name, 'Current', 'uA', highFreqPWM=highFreqPWM, pinOutVoltsMax=pinsVoltsMax, pinInVoltsMax=pinsVoltsMax)
        
        SysCompArduino.__init__(self, name,
                                (voltageVar, currentVar, varEnable),
                                'monitor/change High Voltage variables')
        
    def FirstTimeOnline(self):
        self.GetController().PinModeOut(self.vars['enable'].pin)
        super(Spellman, self).FirstTimeOnline()
        
    def PreEditV(self, value):
        pass
        

class SpellmanCZE30PN2000(Spellman):
    def __init__(self, name, pinEnable, pinPolaritySet, pinPolarityPositive, pinInV, pinInI, pinOutV, pinOutI=None, highFreqPWM=False, safetyMaxAbsVoltage=None):
        self.varPolaritySet = SysVarDigitalArduino('polaritySet', pinPolaritySet, name)
        self.varPolarityPositive = SysVarDigitalArduino('polarityPositive', pinPolarityPositive, name, editable=False)
        rangeV = [-30, 30]
        rangeI = [0, 300]
        Spellman.__init__(self, name, rangeV, rangeI, pinEnable, pinInV, pinInI, pinOutV, pinOutI, highFreqPWM=highFreqPWM, pinsVoltsMax=5, safetyMaxAbsVoltage=safetyMaxAbsVoltage)
        
    def SetPolarityPositive(self, positive=True):
        self.varPolaritySet.Set('on' if positive else 'off');
    
    def GetPolarityPositive(self):
        return self.varPolarityPositive.Get() == 'on'

    def PreEditV(self, value):
        ''' function overload '''
        self.varPolaritySet.Set('on' if value > 0 else 'off');

    def FirstTimeOnline(self):
        self.GetController().PinModeOut(self.varPolaritySet.pin)
        self.GetController().PinModeIn(self.varPolarityPositive.pin)
        super(SpellmanCZE30PN2000, self).FirstTimeOnline()


class SpellmanUM20_4_PLUS(Spellman):
    def __init__(self, name, pinEnable, pinInV, pinInI, pinOutV, pinOutI=None, highFreqPWM=False, safetyMaxAbsVoltage=None):
        rangeV = [0, 20]
        rangeI = [0, 200]
        Spellman.__init__(self, name, rangeV, rangeI, pinEnable, pinInV, pinInI, pinOutV, pinOutI, highFreqPWM=highFreqPWM, pinsVoltsMax=4.64, safetyMaxAbsVoltage=safetyMaxAbsVoltage)


class SpellmanUM20_4_MINUS(Spellman):
    def __init__(self, name, pinEnable, pinInV, pinInI, pinOutV, pinOutI=None, highFreqPWM=False, safetyMaxAbsVoltage=None):
        rangeV = [-20, 0]
        rangeI = [0, 200]
        Spellman.__init__(self, name, rangeV, rangeI, pinEnable, pinInV, pinInI, pinOutV, pinOutI, highFreqPWM=highFreqPWM, pinsVoltsMax=4.64, safetyMaxAbsVoltage=safetyMaxAbsVoltage)


from __future__ import division
from instrumentino.controllers.arduino import SysCompArduino,\
    SysVarAnalogArduinoUnipolar
__author__ = 'yoelk'

class ParkerPressureController(SysCompArduino):
    def __init__(self, name, rangeP, pinInP, pinOutP, highFreqPWM=False):
        SysCompArduino.__init__(self, name,
                                (SysVarAnalogArduinoUnipolar('P', rangeP, pinInP, pinOutP, name, 'Pressure', 'psi', highFreqPWM=highFreqPWM),),
                                'monitor/change pressure')


from __future__ import division
__author__ = 'yoelk'

from instrumentino import cfg
import wx
from wx import xrc
import wx.lib.masked as  masked
import time
from collections import OrderedDict

class SysVar(object):
    '''
    A system variable
    
    Sub-classes should implement:
    - Update()         -    Update the display panel and the log graph for this variable
    - GetFunc()        -    read the value of the variable from the hardware
    - SetFunc(value)   -    Set the value of the variable in the hardware
    - OnEdit(event)    -    called when the display panel of the variable is edited
    '''
    def __init__(self, name, controllerClass, compName='', helpLine='', editable=True, PreSetFunc=None, PostGetFunc=None):
        self.name = name
        self.compName = compName
        self.controllerClass = controllerClass
        self.helpLine = helpLine
        self.editable = editable
        self.widget = None
        self.PreSetFunc = PreSetFunc
        self.PostGetFunc = PostGetFunc

    def Get(self):
        '''
        Call the sub-class' GetFunc()
        '''
        value = self.GetFunc()
        if self.PostGetFunc != None:
            self.PostGetFunc(value)
        return value
    
    def Set(self, value):
        '''
        Call the sub-class' SetFunc()
        '''
        if self.editable:
            if self.PreSetFunc != None:
                self.PreSetFunc(value)                
            self.SetFunc(value)
            
    def Update(self):
        value = self.Get()
        self.UpdatePanel(value)
        cfg.logGraph.AddData(self.FullName(), value)
            
    def FullName(self):
        unitsStr = ' (' + self.units + ')' if hasattr(self, 'units') else ''
        compStr = self.compName + ': ' if self.compName != '' else '' 
        return compStr + self.name + unitsStr
    
    def GetController(self):
        '''
        Get the working instance of the appropriate controller
        '''
        return cfg.GetController(self.controllerClass)

class SysVarAnalog(SysVar):
    '''
    An analog variable (float), represented by a wx.TextCtrl on the display
    
    Sub-classes should implement:
    - GetFunc()        -    read the value of the variable from the hardware
    - SetFunc(value)   -    Set the value of the variable in the hardware
    '''
    def __init__(self, name, range, controllerClass, compName='', helpLine='', editable=True, units='', PreSetFunc=None, PostGetFunc=None):
        SysVar.__init__(self, name, controllerClass, compName, helpLine, editable, PreSetFunc, PostGetFunc)
        self.units = units
        self.range = range
        self.monitorTextCtrl = None
        self.editTextCtrl = None
        
    def UpdatePanel(self, value):
        '''
        Update the variable's panel with newly read data
        '''
        if self.monitorTextCtrl != None and value != None:
            self.monitorTextCtrl.ChangeValue(value)

    def OnEdit(self, event):
        '''
        Set the variable's value from the user's input
        '''
        keycode = event.GetKeyCode()
        if keycode == wx.WXK_RETURN or keycode == wx.WXK_NUMPAD_ENTER:
            value = float(event.GetEventObject().GetValue())
            if self.range[0] <= value <= self.range[1]:
                self.Set(value)
        event.Skip()
        
    def CreatePanel(self, parent):
        '''
        Create a panel to represent the variable
        '''
        rows = 2 if self.editable else 1
        sizer = wx.FlexGridSizer(rows=rows, cols=2)
        panel = wx.Panel(parent)
        panel.SetSizer(sizer)
        sizer.Add(wx.StaticText(panel, label=self.name + ' (' + self.units + '):'))
        self.monitorTextCtrl = masked.NumCtrl(panel,
                                              value=0,
                                              integerWidth=cfg.numIntegerPartWidth,
                                              fractionWidth=cfg.numFractionPartWidth,
                                              min=self.range[0],
                                              max=self.range[1],
                                              style=wx.TE_CENTRE | wx.TE_READONLY) 
        sizer.Add(self.monitorTextCtrl)
        if self.editable:
            sizer.Add(wx.StaticText(panel, label='Set to:'))
            editTextCtrl = masked.NumCtrl(panel,
                                          value=0,
                                          integerWidth=cfg.numIntegerPartWidth,
                                          fractionWidth=cfg.numFractionPartWidth,
                                          min=self.range[0],
                                          max=self.range[1],
                                          style=wx.TE_CENTRE | wx.WANTS_CHARS)
            editTextCtrl.Bind(wx.EVT_KEY_DOWN, self.OnEdit)
            sizer.Add(editTextCtrl)

        return panel
            

class SysVarDigital(SysVar):
    '''
    A digital variable (multiple choice), represented by a Set of wx.RadioButtons on the display
    
    Sub-classes should implement:
    - GetFunc()        -    read the value of the variable from the hardware
    - SetFunc(value)   -    Set the value of the variable in the hardware
    '''
    def __init__(self, name, states, controllerClass, compName='', helpLine='', editable=True, PreSetFunc=None, PostGetFunc=None):
        SysVar.__init__(self, name, controllerClass, compName, helpLine, editable, PreSetFunc, PostGetFunc)
        self.radioButtons = None
        self.states = states
        
    def UpdatePanel(self, value):
        '''
        Update the variable's panel with newly read data
        '''
        if self.radioButtons != None and value != None:
            try:
                for rbName, rb in self.radioButtons.items():
                    rb.SetValue(rbName == value)
            except:
                pass
    
    def OnEdit(self, event):
        '''
        Set the variable's value from the user's input
        '''
        self.Set(event.GetEventObject().GetLabel())

    def CreatePanel(self, parent):
        '''
        Create a panel to represent the variable
        '''
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        panel = wx.Panel(parent)
        panel.SetSizer(sizer)
        self.radioButtons = {}
        sizer.Add(wx.StaticText(panel, label=self.name + ':'))
        for state in self.states:
            radioButton = wx.RadioButton(panel, label=state)
            sizer.Add(radioButton)
            self.radioButtons[state] = radioButton
            panel.Bind(wx.EVT_RADIOBUTTON, self.OnEdit, radioButton)

        return panel

    
class SysComp(object):
    '''
    A system component, which has variables, and is represented on the screen as a panel containing the variables' panels
    '''
    def __init__(self, name, vars, controllerClass, helpLine=''):
        self.name = name
        self.controllerClass = controllerClass
        self.helpLine = helpLine
        self.vars = OrderedDict([(var.name, var) for var in vars])
        self.panel = None
        self.online = False
        
    def CreatePanel(self, parent):
        '''
        Create a panel for the component, based on its variables
        
        Sub-classes may implement FirstTimeOnline() to do some initialization when communication is established
        '''
        vars = self.vars.values()
        if len(vars) == 0:
            return None
        
        self.panel = wx.Panel(parent)
        staticBox = wx.StaticBox(self.panel, label=self.name)
        sizer = wx.StaticBoxSizer(staticBox, wx.VERTICAL)
        self.panel.SetSizer(sizer)
        for var in vars[0:-1]:
            sizer.Add(var.CreatePanel(self.panel))
            sizer.Add(wx.StaticLine(self.panel), flag=wx.EXPAND)
        sizer.Add(vars[-1].CreatePanel(self.panel))

        return self.panel
    
    def Update(self):
        '''
        Update all variables
        '''
        if self.online == False:
            self.online = True
            self.FirstTimeOnline()

        if self.panel != None:
            for var in self.vars.values():
                var.Update()

    def FirstTimeOnline(self):
        '''
        Called when communication is first setup. To be implemented by subclass.
        '''
        pass
                
    def Enable(self, isEnabled):
        '''
        Enable/disable the availability of the panel on the screen
        '''
        if self.panel != None:
            self.panel.Enable(isEnabled)
            
    def GetController(self):
        '''
        Get the working instance of the appropriate controller
        '''
        return cfg.GetController(self.controllerClass)


from __future__ import division
from instrumentino.controllers.arduino import SysCompArduino,\
    SysVarAnalogArduinoUnipolar
__author__ = 'yoelk'

class PewatronPressureSensor(SysCompArduino):
    def __init__(self, name, rangeP, pinInP):
        SysCompArduino.__init__(self, name,
                                (SysVarAnalogArduinoUnipolar('P', rangeP, pinInP, None, name, 'Pressure', 'mbar', pinInVoltsMax=4.5, pinInVoltsMin=0.5),),
                                'monitor pressure')


from __future__ import division
from instrumentino.comp import SysVarDigital, SysVarAnalog, SysComp
from instrumentino.controllers import InstrumentinoController
__author__ = 'yoelk'

from instrumentino import cfg
import wx
import serial
import time
from threading import Semaphore

class Arduino(InstrumentinoController):
    '''
    This class implements an interface to a simple arduino program that allows control of pins and softSerial.
    The idea for this approach was taken from this code: https://github.com/vascop/Python-Arduino-Proto-API-v2 
    '''

    # Arduino constants
    HIGH    = 1
    LOW     = 0

    PIN_VOLT_MIN        = 0
    PIN_VOLT_MAX        = 5
    PIN_VOLT_RANGE      = (PIN_VOLT_MAX - PIN_VOLT_MIN)
    PIN_VOLT_MIDDLE     = (PIN_VOLT_MIN + PIN_VOLT_RANGE / 2)

    ANAL_OUT_VAL_MAX    = 255
    ANAL_OUT_VAL_MIN    = 0
    ANAL_OUT_VAL_RANGE  = (ANAL_OUT_VAL_MAX - ANAL_OUT_VAL_MIN)
    ANAL_OUT_VAL_MIDDLE = (ANAL_OUT_VAL_MIN + ANAL_OUT_VAL_RANGE / 2)

    ANAL_IN_VAL_MAX     = 1023
    ANAL_IN_VAL_MIN     = 0
    ANAL_IN_VAL_RANGE   = (ANAL_IN_VAL_MAX - ANAL_IN_VAL_MIN)
    
    baudrate=115200
    serialTimeoutSec=0.05
    serialWriteTimeoutSec=0
    # the read delay should to at least twice the time for a single read, so there's also time to write
    cacheReadDelayMilisec = max(250, (serialTimeoutSec + serialWriteTimeoutSec)*1000*2)
    
    pinValuesCache = {}
    timer = None
    
    name = 'Arduino'
    
    def __init__(self):
        '''
        init 
        '''
        InstrumentinoController.__init__(self, self.name)
        self.accessSemaphore = Semaphore()
        self.serial = None
    
    def Connect(self, port):
        '''
        port - the name of the serial port connected to the Arduino. This works on unix and windows systems, given the appropriate name
        '''
        try:
            self.serial = serial.Serial(port, self.baudrate, writeTimeout=self.serialWriteTimeoutSec, timeout=self.serialTimeoutSec)
        except:
            cfg.LogFromOtherThread('Arduino did not respond', True)
            return None

        time.sleep(2)
        
        if self.serial == None or self._sendData('Read A0') == None:
            return None
        
        # start reading timer
        self.timer = wx.Timer(cfg.app)
        cfg.app.Bind(wx.EVT_TIMER, self.CacheUpdate, self.timer)
        self.timer.Start(self.cacheReadDelayMilisec)
        
        return True

    def Close(self, reset=True):
        '''
        Close the connection to the Arduino
        '''
        if self.serial != None:
            if self.timer != None:
                self.timer.Stop()
            if reset:
                self.Reset()
            self.serial.close()
            self.serial = None

    def CacheUpdate(self, event):
        pins = ''
        # save the keys in case they change while we read
        keys = self.pinValuesCache.keys()
        for k in keys:
            pins += k + ' '
        
        valuesStr = self._sendData('Read %s'%(pins.strip()))
        if valuesStr == None:
            return
        
        values = valuesStr.split(' ')
        
        # values are received in the same order we asker for them
        for key, val in zip(keys, values):
            self.pinValuesCache[key] = int(val)
            
    def PinModeOut(self, pin):
        '''
        Set digital pin mode as output 
        '''
        self.PinMode(pin, True)

    def PinModeIn(self, pin):
        '''
        Set digital pin mode as input 
        '''
        self.PinMode(pin, False)

    def PinMode(self, pin, modeOut):
        '''
        Set digital pin mode
        modeOut - set to 'out' if True, else set to 'in'  
        '''
        self._sendData('Set %d %s'%(pin, 'out' if modeOut else 'in'), wait=True)

    def AnalogWriteVolts(self, pin, volts):
        '''
        Set the voltage level (using PWM) of a digital output pin directly
        volts - between 0-5 volts  
        '''
        self.AnalogWrite(pin, self.ANAL_OUT_VAL_MAX *  volts / self.PIN_VOLT_MAX)

    def AnalogWriteFraction(self, pin, fraction, maxV=5, minV=0):
        '''
        Set the voltage level (using PWM) of a digital output pin using a fraction
        fraction - between 0-1  
        minV - minimal voltage
        maxV - maximal voltage
        '''
        self.AnalogWriteVolts(pin, minV + (maxV - minV) * fraction)

    def AnalogWrite(self, pin, value):
        '''
        Set the voltage level (using PWM) of a digital output pin using an 8-bit value
        value - between 0-255  
        '''
        self._sendData('Write %d anal %d'%(pin, value), wait=True)
            
    def SetHighFreqPwm(self, pin):
        '''
        Set this PWM pin to work in high frequency mode   
        '''
        self._sendData('SetPwmFreq %d 1'%(pin), wait=True)

    def AnalogReadVolts(self, pin):
        '''
        Read the voltage level of an analog input pin directly
        returns - value between 0-5 (volts)  
        '''
        value = self.AnalogRead(pin)
        if value != None:
            return self.PIN_VOLT_MAX * value / self.ANAL_IN_VAL_MAX
    
    def AnalogReadFraction(self, pin, maxV=5, minV=0):
        '''
        Read the voltage level of an analog input pin using a fraction
        minV - minimal voltage
        maxV - maximal voltage
        returns - value between 0-1  
        '''
        value = self.AnalogReadVolts(pin)
        if value != None:
            return (value - minV) / (maxV - minV)

    def AnalogRead(self, pin):
        '''
        Read the voltage level of an analog input pin using a 10-bit value
        
        Returns: value between 0-1023  
        '''
        # Get value from cache if possible, and if not, add it to the wish-list
        try:
            return self.pinValuesCache['A' + str(pin)]
        except KeyError:
            self.pinValuesCache['A' + str(pin)] = 0

    def DigitalWriteHigh(self, pin):
        '''
        Set the voltage level of a digital output pin to the maximum (HIGH level)
        '''
        self.DigitalWrite(self, pin, 1)

    def DigitalWriteLow(self, pin):
        '''
        Set the voltage level of a digital output pin to the minimum (LOW level)
        '''
        self.DigitalWrite(self, pin, 0)

    def DigitalWrite(self, pin, value):
        '''
        Set the voltage level of a digital output pin to value (1 to HIGH and 0 to low)
        '''
        self._sendData('Write %d digi %d'%(pin, value), wait=True)

    def DigitalRead(self, pin):
        '''
        Read the voltage level of a digital pin
        
        Returns: 1 for HIGH and 0 for LOW
        '''
        # Get value from cache if possible, and if not, add it to the wish-list
        try:
            return self.pinValuesCache['D' + str(pin)]
        except KeyError:
            self.pinValuesCache['D' + str(pin)] = 0
    
    def HardSerConnect(self, baudrate, port=1):
        '''
        Setup a hardware serial port for communication
        baudrate - the serial connection baudrate
        port - the hardware serial port number
        '''
        self._sendData('HardSerConnect %d %d'%(baudrate, port), wait=True)
        time.sleep(1)
    
    def SoftSerConnect(self, pinRx, pinTx, baudrate, port=1):
        '''
        Setup a software serial port using pinRx and pinTx for communication
        baudrate - the serial connection baudrate (should be much smaller than the baudrate used for the Arduino)
        port - a port number to identify future transactions
        '''
        self._sendData('SoftSerConnect %d %d %d %d'%(pinRx, pinTx, baudrate, port), wait=True)
        time.sleep(1)
    
    def SerSend(self, serialData, writeTimeoutSec=None, isSoftSerial=True, port=1):
        '''
        Send serial data over an open serial connection
        serialData - the data to send. a NULL ('\0') character is added at the end to signify the end of transmission
        writeTimeoutSec - time in seconds to wait for a software serial response
        isSoftSerial - True for software serial ports, False for hardware serial ports
        port - the port number to use
        
        Returns: the data received (if any)
        '''        
        self.accessSemaphore.acquire(True)
        if writeTimeoutSec != None:
            self._sendData('SerReceive %s %d'%('soft' if isSoftSerial else 'hard', port), lock=False)
        
        self._sendData('SerSend %s %d'%('soft' if isSoftSerial else 'hard', port), lock=False)
        self._sendData(serialData + '\0', False, lock=False)
        
        rxData = None
        if writeTimeoutSec != None:
            time.sleep(writeTimeoutSec)
            rxData = self._sendData('SerReceive %s %d'%('soft' if isSoftSerial else 'hard', port), lock=False)
            
        self.accessSemaphore.release()
        return rxData

    def Reset(self):
        '''
        Reset the mode of all digital pins to input
        '''
        self._sendData('Reset', wait=True)
        time.sleep(1)
    
    def _sendData(self, txData, addLineBreak=True, lock=True, wait=False, log=False):
        '''
        This function should be called only in this class
        Send a command (txData) to the Arduino and wait for acknowledgment.
        Failure of the acknowledgment to arrive logs the event and raises an exception
         
        writeTimeoutSec - time in seconds to wait for a software serial response
        addLineBreak - add a line break after txData (to tell the Arduino to execute the command) 
        lock - lock access to Arduino during this function
        wait - wait until the Arduino is unlocked. This is used for critical commands such as write commands
        log - log the communication to the screen. mostly for debug purposes
        
        Returns: the data received (if any)
        '''
        if self.serial == None:
            return None
                
        if lock == True:
            if self.accessSemaphore.acquire(False) == False:
                if wait:
                    while self.accessSemaphore.acquire(False) == False:
                        pass
                else:
                    return None
        
        if log:
            print 'we say: ' + txData
        if addLineBreak == True:
            txData = txData + '\r'

        try:
            self.serial.write(txData)
            rxData = self._getData()
        except:
            rxData = ''
        
        if log:
            print 'Arduino says: ' + rxData
        
        answerEnd = rxData.rfind("done") 
        if answerEnd == -1:
            cfg.LogFromOtherThread('Arduino did not respond', True)
            return None

        if lock == True:
            self.accessSemaphore.release()
            
        return rxData[0:answerEnd].strip();

    def _getData(self):
        '''
        This function should be called only in the class
        Reads the Arduino response for the last command
        
        Returns: the response
        '''
        try:        
            return self.serial.read(300)
        except:
            return ''
    
    
# base class and variables
class SysVarDigitalArduino(SysVarDigital):
    def __init__(self, name, pin, compName='', stateToValue={'on': 1, 'off':0}, helpLine='', editable=True, PreSetFunc=None):
        self.stateToValue = stateToValue
        self.valueToState = {v: k for k, v in stateToValue.items()}
        SysVarDigital.__init__(self, name, self.stateToValue.keys(), Arduino, compName, helpLine, editable, PreSetFunc)
        self.pin = pin

    def FirstTimeOnline(self):
        if self.pin != None:
            self.GetController().PinMode(self.pin, self.editable)
        
    def GetFunc(self):
        value = self.GetController().DigitalRead(self.pin)
        return self.valueToState[value] if value != None else None
    
    def SetFunc(self, state):
        self.GetController().DigitalWrite(self.pin, self.stateToValue[state])


class SysVarAnalogArduino(SysVarAnalog):
    '''
    An Arduino analog variable
    '''
    def __init__(self, name, range, pinAnalIn, pinPwmOut=None, SetPolarityPositiveFunc=None, GetPolarityPositiveFunc=None, compName='', helpLine='', units='', PreSetFunc=None, highFreqPWM=False, pinOutVoltsMax=5, pinInVoltsMax=5, pinOutVoltsMin=0, pinInVoltsMin=0, PostGetFunc=None):
        SysVarAnalog.__init__(self, name, range, Arduino, compName, helpLine, pinPwmOut != None, units, PreSetFunc, PostGetFunc)
        self.pinIn = pinAnalIn
        self.pinOut = pinPwmOut
        self.SetPolarityPositiveFunc = SetPolarityPositiveFunc
        self.GetPolarityPositiveFunc = GetPolarityPositiveFunc
        self.highFreqPWM = highFreqPWM
        self.pinOutVoltsMax = pinOutVoltsMax
        self.pinInVoltsMax = pinInVoltsMax
        self.pinOutVoltsMin = pinOutVoltsMin
        self.pinInVoltsMin = pinInVoltsMin
        
    def FirstTimeOnline(self):
        if self.pinOut != None:
            self.GetController().PinModeOut(self.pinOut)
            if self.highFreqPWM:
                self.GetController().SetHighFreqPwm(self.pinOut)
    
    def GetUnipolarRange(self):
        return self.GetUnipolarMax() - self.GetUnipolarMin()
    
    def GetFunc(self):
        fraction = self.GetController().AnalogReadFraction(self.pinIn, self.pinInVoltsMax, self.pinInVoltsMin)
        sign = 1 if self.GetPolarityPositiveFunc() else -1
        return sign * (self.GetUnipolarMin() + (self.GetUnipolarRange() * fraction)) if fraction != None else None
    
    def SetFunc(self, value):
        self.GetController().AnalogWriteFraction(self.pinOut, (abs(value) - self.GetUnipolarMin()) / self.GetUnipolarRange(), self.pinOutVoltsMax, self.pinOutVoltsMin)    

class SysVarAnalogArduinoUnipolar(SysVarAnalogArduino):
    '''
    A unipolar analog variable, for which the range has to be [X1,X2] or [-X1,-X2].
    The voltage on the pin (normally 0-5 V) corresponds percentage-wise to the variable's value between X1 and X2 (or -X1 and -X2).
    '''
    def __init__(self, name, range, pinAnalIn, pinPwmOut, compName='', helpLine='', units='', PreSetFunc=None, highFreqPWM=False, pinOutVoltsMax=5, pinInVoltsMax=5, pinOutVoltsMin=0, pinInVoltsMin=0, PostGetFunc=None):
        SysVarAnalogArduino.__init__(self, name, range, pinAnalIn, pinPwmOut, self.SetPolarityPositiveFunc, self.GetPolarityPositiveFunc, compName, helpLine, units, PreSetFunc, highFreqPWM, pinOutVoltsMax, pinInVoltsMax, pinOutVoltsMin, pinInVoltsMin, PostGetFunc)
        self.sign = 1 if range[0] >=0 and range[1] >= 0 else -1

    def SetPolarityPositiveFunc(self):
        pass
    
    def GetPolarityPositiveFunc(self):
        return self.sign == 1

    def GetUnipolarMin(self):
        return min(abs(self.range[0]), abs(self.range[1]))
    
    def GetUnipolarMax(self):
        return max(abs(self.range[0]), abs(self.range[1]))
    
class SysVarAnalogArduinoBipolarWithExternalPolarity(SysVarAnalogArduino):
    '''
    A bipolar symmetric analog variable, for which the range has to be [-X,X].
    The voltage on the pin (normally 0-5 V) corresponds percentage-wise to the variable's absolute value between 0 and X (or -X).
    Polarity is set and read by user specific functions
    '''
    def __init__(self, name, range, pinAnalIn, pinPwmOut, SetPolarityPositiveFunc, GetPolarityPositiveFunc, compName='', helpLine='', units='', PreSetFunc=None, highFreqPWM=False, pinOutVoltsMax=5, pinInVoltsMax=5, pinOutVoltsMin=0, pinInVoltsMin=0, PostGetFunc=None):
        SysVarAnalogArduino.__init__(self, name, range, pinAnalIn, pinPwmOut, SetPolarityPositiveFunc, GetPolarityPositiveFunc, compName, helpLine, units, PreSetFunc, highFreqPWM, pinOutVoltsMax, pinInVoltsMax, pinOutVoltsMin, pinInVoltsMin, PostGetFunc)

    def GetUnipolarMin(self):
        return 0
    
    def GetUnipolarMax(self):
        # it abs(range[0]) and abs(range[1]) should be equal
        return abs(self.range[1])


class SysCompArduino(SysComp):
    def __init__(self, name, vars, helpLine=''):
        SysComp.__init__(self, name, vars, Arduino, helpLine)
        
    def FirstTimeOnline(self):
        for var in self.vars.values():
            var.FirstTimeOnline()


from __future__ import division
from instrumentino.controllers.arduino import SysCompArduino,\
    SysVarAnalogArduinoUnipolar, SysVarDigitalArduino
__author__ = 'yoelk'

from instrumentino import cfg

class MKSMassFlowController(SysCompArduino):
    def __init__(self, name, pinInPercent, pinOutPercent, pinOutClose=None):
        SysCompArduino.__init__(self, name, 
                                (SysVarAnalogArduinoUnipolar('Flow', [0,100], pinInPercent, pinOutPercent, name, 'Flow percentage', '%', self.PreEditPercent, highFreqPWM=True),),
                                'monitor/change gas flow')
        
        self.pinOutClose = pinOutClose
        if self.pinOutClose != None:
            self.varClose = SysVarDigitalArduino('close', pinOutClose, name)
        
    def FirstTimeOnline(self):
        if self.pinOutClose != None:
            self.GetController().PinModeOut(self.varClose.pin)
        return super(MKSMassFlowController, self).FirstTimeOnline()
        
    def PreEditPercent(self, value):
        if self.pinOutClose != None:
            if value == 0:
                self.varClose.Set('off')
            else:
                self.varClose.Set('on')


from __future__ import division
__author__ = 'yoelk'

from instrumentino import cfg
import wx
from wx import xrc
from threading import Thread
import sys
import copy
from datetime import datetime
from collections import OrderedDict

from wx.lib.agw import ultimatelistctrl as ULC

class ExecutableListCtrl(object):
    '''
    A general list with add, delete and Run buttons. The same infrastructure for the actions' list and the methods' list
    '''
    def __init__(self, parent, panel, columnNumToName, runStartString, listDataItems=[]):
        self.columnNumToName = OrderedDict([(0, '#')] + columnNumToName.items())
        self.columnNameToNum = {v: k for k, v in self.columnNumToName.items()}
        self.runStartString = runStartString
        self.panel = panel
        self.runButton = (xrc.XRCCTRL(parent, 'runButton'))
        self.addButton = (xrc.XRCCTRL(parent, 'addButton'))
        self.removeButton = (xrc.XRCCTRL(parent, 'removeButton'))
        self.list = ULC.UltimateListCtrl(self.panel, agwStyle=wx.LC_REPORT 
                                         | wx.LC_VRULES
                                         | wx.LC_HRULES
                                         | ULC.ULC_HAS_VARIABLE_ROW_HEIGHT
                                         | ULC.ULC_SINGLE_SEL)
        self.panel.GetSizer().Add(self.list, 1, wx.EXPAND|wx.ALL|wx.GROW)
        for idx, name in self.columnNumToName.items():
            columnInfo = ULC.UltimateListItem()
            columnInfo._mask = wx.LIST_MASK_TEXT
            columnInfo._text = name
            self.list.InsertColumnInfo(idx, columnInfo)
        
        if len(listDataItems) == 0:
            listDataItems = [self.getDefaultDataItem(),]
        self.populateList(listDataItems)
        self.listUpdate()
        
        parent.Bind(wx.EVT_BUTTON, self.onAddButton, id=xrc.XRCID('addButton'))
        parent.Bind(wx.EVT_BUTTON, self.onRemoveButton, id=xrc.XRCID('removeButton'))
        parent.Bind(wx.EVT_BUTTON, self.onRunButton, id=xrc.XRCID('runButton'))
        self.list.Bind(ULC.EVT_LIST_KEY_DOWN, self.onKeyPress)       

    def populateList(self, listDataItems):
        '''
        Populate the list with items. Delete old ones.
        '''
        self.list.Freeze()
        # Clear old list        
        item = self.list.GetNextItem(-1)
        while item != -1:
            self.removeListDataItem(item)
            item = self.list.GetNextItem(-1)
        
        # Add all elements
        for _, listDataItem in enumerate(listDataItems):
            self.addListDataItem(listDataItem)
                        
        self.list.Thaw()
        self.listUpdate()

    def listUpdate(self):
        '''
        Refresh the list on screen
        '''
        for idx in self.columnNumToName.keys():
            self.list.SetColumnWidth(idx, wx.LIST_AUTOSIZE)
        
        self.list.Update()
        self.panel.Refresh()

    def addListDataItem(self, listDataItem=None):
        '''
        Add a list item
        '''
        if listDataItem == None:
            listDataItem = self.getDefaultDataItem()
        if listDataItem == None:
            return
         
        insertIndex = self.list.GetFirstSelected() if self.list.GetFirstSelected() != -1 else sys.maxint
        index = self.list.InsertStringItem(insertIndex, '')
        self.list.SetStringItem(index, 0, str(index + 1))
        for col in self.columnNumToName.keys()[1:]:
            self.list.SetStringItem(index, col, '')
         
        item = self.list.GetItem(index, 1)
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        panel = wx.Panel(self.list)
        panel.SetSizer(sizer)
        panel.SetName(str(index + 1))
        sizer.Add(self.getFirstColumnWidget(panel, listDataItem), flag=wx.EXPAND)
        item.SetWindow(panel, expand=True)
        self.list.SetItem(item)
        self.list.SetItemData(index, [listDataItem,])
        
        self.setOtherColumns(index, listDataItem)
        
        self.listUpdate()
        self.updateListIndices()
     
    def removeListDataItem(self, index):
        '''
        Remove a list item
        '''
        if index != -1:
            self.list.Freeze()
            self.list.DeleteItem(index)
            
            self.updateListIndices()
            
            self.list.Thaw()
            self.listUpdate()
    
    def updateListIndices(self):
        '''
        Update the indices in the first column. Used when an item is deleted from the middle.
        '''
        index = self.list.GetNextItem(-1)
        if self.list.GetNextItem(index) != -1:
            while index != -1:
                self.list.SetStringItem(index, 0, str(index + 1))
                self.list.GetItemWindow(index, 1).SetName(str(index + 1))
                index = self.list.GetNextItem(index)
    
    def onAddButton(self, evt):
        '''
        User pressed the add button
        '''
        self.addListDataItem()
        
    def onRemoveButton(self, evt):
        '''
        User pressed the remove button
        '''
        self.removeListDataItem(self.list.GetFirstSelected())
    
    def onRunButton(self, evt):
        '''
        User pressed the Run button
        '''
        worker = RunListThread(self.list, self.runStartString)
        worker.start()
    
    def onKeyPress(self, event):
        '''
        User pressed a key while in the list area
        '''
        keycode = event.GetKeyCode()
        if keycode == wx.WXK_DELETE:
            self.removeListDataItem(self.list.GetFirstSelected())
        if keycode == ord('+'):
            self.addListDataItem()
        event.Skip()
        
    def getDataItemsList(self):
        '''
        Return the list's items
        '''
        listDataItems = []
        item = self.list.GetNextItem(-1)
        while item != -1:
            listDataItems.append(self.list.GetItemData(item)[0])
            item = self.list.GetNextItem(item)
            
        return listDataItems
    
    def getDefaultDataItem(self):
        '''
        Return an empty item. To be implemented by subclass.
        '''
        pass
     
    def getFirstColumnWidget(self, panel, listDataItem):
        '''
        Return the widget for the first column. To be implemented by subclass.
        '''
        pass
     
    def setOtherColumns(self, index, action):
        '''
        Set the other columns' content. To be implemented by subclass.
        '''
        pass
        

class RunnableItem(object):
    '''
    An item that can be Run (like an action or a method file)
    '''
    def Run(self):
        '''
        Run. To be implemented by subclass.
        '''
        pass

        
class RunListThread(Thread):
    '''
    Thread class that executes the items in a list
    '''
    def __init__(self, theList, runStartString='*** Run start ***'):
        Thread.__init__(self)
        self.runStartString = runStartString
        self.list = theList
 
    def getDataItemsAndItems(self):
        '''
        Return a double list of items and data items
        '''
        dataItemsAndItems = []
        item = self.list.GetNextItem(-1)
        while item != -1:
            dataItemsAndItems.append((self.list.GetItemData(item)[0], item))
            item = self.list.GetNextItem(item)
            
        return dataItemsAndItems
    
    def run(self):
        '''
        Run the list's items
        '''
        cfg.UpdateControlsFromOtherThread(True)
        cfg.LogFromOtherThread(self.runStartString)
        dataItemsAndItems = self.getDataItemsAndItems()
        for dataItem, item in dataItemsAndItems:
            self.list.Select(item, False)
            
        for dataItem, item in dataItemsAndItems:
            self.list.Select(item, True)
            dataItem.Run()
            if cfg.userStopped:
                cfg.userStopped = False
                break
            self.list.Select(item, False)
        
        cfg.UpdateControlsFromOtherThread(False)
        cfg.LogFromOtherThread('')


from __future__ import division
from instrumentino.controllers.arduino import SysCompArduino, SysVarDigitalArduino
__author__ = 'yoelk'

class DigitalPins(SysCompArduino):
    def __init__(self, name, digiVars=()):
        SysCompArduino.__init__(self, name, digiVars, "turn on/off Arduino digital pins")
 
 
class AnalogPins(SysCompArduino):
    def __init__(self, name, analogVars=()):
        SysCompArduino.__init__(self, name, analogVars, "set Arduino analog pins")


from __future__ import division
__author__ = 'yoelk'

import sys
import os
import serial
import glob
from serial.serialutil import SerialException
try:
    import _winreg as winreg
except ImportError:
    pass
import itertools

class Chdir:    
    '''
    Instantiating this class changes the current directory until the object is deleted
    '''     
    def __init__( self, newPath ):  
        self.savedPath = os.getcwd()
        os.chdir(newPath)

    def __del__( self ):
        os.chdir( self.savedPath )


"""
Lists the serial ports available on the computer.
some of the code was taken from Eli Bendersky (eliben@gmail.com), License: this code is in the public domain
"""
class SerialUtil():
    def enumerate_serial_ports(self):
        """ Uses the Win32 registry to return an
            iterator of serial (COM) ports
            existing on this computer.
        """
        path = 'HARDWARE\\DEVICEMAP\\SERIALCOMM'
        try:
            key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, path)
        except WindowsError:
            raise IterationError
    
        for i in itertools.count():
            try:
                val = winreg.EnumValue(key, i)
                yield str(val[1])
            except EnvironmentError:
                break
            
    def full_port_name(self, portname):
        """ Given a port-name (of the form COM7,
            COM12, CNCA0, etc.) returns a full
            name suitable for opening with the
            Serial class.
        """
        m = re.match('^COM(\d+)$', portname)
        if m and int(m.group(1)) < 10:
            return portname
        return '\\\\.\\' + portname
    
    def getSerialPortsList(self):
            ports = []
            if os.name == 'nt':
                for portname in self.enumerate_serial_ports():
                    ports.append(portname)
            elif os.name == 'posix':
                ports = glob.glob('/dev/tty.*')
                
            return ports
        
if __name__ == '__main__':
    print SerialUtil().getSerialPortsList()


from __future__ import division
from instrumentino.controllers.arduino import SysCompArduino, SysVarDigitalArduino,\
    SysVarAnalogArduinoUnipolar
__author__ = 'yoelk'

from instrumentino import cfg

class HvmSubMiniHighVoltage(SysCompArduino):
    def __init__(self, name, rangeV, rangeI, pinEnable, activeHigh, pinInV, pinInI, pinOutV, highFreqPWM=False):
        self.activeHigh = activeHigh
        self.varEnable = SysVarDigitalArduino('enable', pinEnable, name)
        SysCompArduino.__init__(self, name,
                                (SysVarAnalogArduinoUnipolar('V', rangeV, pinInV, pinOutV, name, 'Voltage', 'kV', self.PreEditV, highFreqPWM=highFreqPWM, pinInVoltsMax=1),
                                 SysVarAnalogArduinoUnipolar('I', rangeI, pinInI, None, name, 'Current', 'uA', highFreqPWM=highFreqPWM, pinInVoltsMax=1)),
                                'monitor/change High Voltage variables')
        
    def FirstTimeOnline(self):
        self.GetController().PinModeOut(self.varEnable.pin)
        super(HvmSubMiniHighVoltage, self).FirstTimeOnline()
        
    def PreEditV(self, value):
        if value != 0:
            self.varEnable.Set('on' if self.activeHigh else 'off')
        else:
            self.varEnable.Set('off' if self.activeHigh else 'on')

        
class HvmSMHV05100(HvmSubMiniHighVoltage):
    def __init__(self, name, pinEnable, activeHigh, pinInV, pinInI, pinOutV, highFreqPWM=False):
        HvmSubMiniHighVoltage.__init__(self, name, [0, 10], [0, 100], pinEnable, activeHigh, pinInV, pinInI, pinOutV, highFreqPWM=highFreqPWM)

class HvmSMHV05100N(HvmSubMiniHighVoltage):
    def __init__(self, name, pinEnable, activeHigh, pinInV, pinInI, pinOutV, highFreqPWM=False):
        HvmSubMiniHighVoltage.__init__(self, name, [-10, 0], [0, 100], pinEnable, activeHigh, pinInV, pinInI, pinOutV, highFreqPWM=highFreqPWM)


#!/usr/bin/env python
import ez_setup
ez_setup.use_setuptools()
from setuptools import setup, find_packages

setup(install_requires = ['wxPython', 'pyserial', 'matplotlib'], #, 'pylab', 'mpl_toolkits'
      packages=find_packages(),
      package_data={'': ['*.png', '*.xrc', '*.dll']},
      zip_safe=True,

      # metadata for upload to PyPI
      name='instrumentino',
      version = '1.0',
      author = 'Joel Koenka',
      author_email = 'yoelk@tx.technion.ac.il',
      description = "instrumentino is a GUI framework for instruments based Arduino controllers. Extensions for other controllers exist a well.",
      license = 'GPLv3',
      keywords = 'Instrumentino, Arduino',
      url = 'http://www.chemie.unibas.ch/~hauser/index.html',
      # could also include long_description, download_url, classifiers, etc.
      )


from __future__ import division
from instrumentino.controllers.arduino import SysCompArduino, SysVarDigitalArduino
__author__ = 'yoelk'

from instrumentino import cfg

class EdaqEcorder(SysCompArduino):
    def __init__(self, name, pin):
        SysCompArduino.__init__(self, name, (SysVarDigitalArduino('trigger', pin, name, PreSetFunc=self.Pause),), 'trigger the e-corder')

    def Pause(self, value):
        if value == 'off':
            cfg.Sleep(1)
        
    def TriggerPulse(self):
        self.vars['trigger'].Set('on')
        cfg.Sleep(0.01)
        self.vars['trigger'].Set('off')


from __future__ import division
from datetime import datetime
import time
import os
from pkg_resources import resource_filename
import wx

__author__ = 'yoelk'

'''
Instrumentino configuration variables
'''
# The way we show numbers
numIntegerPartWidth = 4
numFractionPartWidth = 3
numStringFormat = '{:' + str(numIntegerPartWidth) + '.' + str(numFractionPartWidth) + 'f}' 

# Files handeled
methodWildcard = 'Method file (*.mtd)|*.mtd'
sequenceWildcard = 'Sequence file (*.seq)|*.seq'

# Events handeled by the main window
EVT_LOG_UPDATE = wx.NewId()
EVT_UPDATE_CONTROLS = wx.NewId()

app = None
mainFrame = None
logTextCtrl = None
logGraph = None
commandsLogFile = None
signalsLogFile = None
systemUid = None

controllers = []
userStopped = False

def InitVariables(arguApp):
    '''
    init system variables
    '''
    global app
    app = arguApp
    
    global mainFrame
    mainFrame = arguApp.mainFrame
    
    global logTextCtrl
    logTextCtrl = wx.xrc.XRCCTRL(arguApp.mainFrame, 'logTextCtrl')
    logTextCtrl.SetEditable(False)
    
    global logGraph
    logGraph = arguApp.logGraph
    
    global timeNow
    timeNow = datetime.now().strftime('%Y_%m_%d-%H_%M_%S')
    
    global commandsLogFile
    commandsLogFile = open(LogPath(timeNow + '.txt'), 'w')
    
    global signalsLogFile
    signalsLogFile = open(LogPath(timeNow + '.csv'), 'w')
    
    global systemUid
    systemUid = arguApp.system.GetSystemUid()

def AddControllerIfNeeded(controllerClass):
    '''
    add a controller to the needed controllers list, only if it doesn't exist already
    '''
    global controllers
    for c in controllers:
        if isinstance(c, controllerClass):
            return
        
    # if reached here, add an object of the given class
    controllers += [controllerClass()]

def Close():
    global controllers
    for c in controllers:
        c.Close()
    
def AllOnline():
    '''
    check if all controllers are online
    '''
    global controllers
    for c in controllers:
        if not c.online:
            return False
        
    # if reached here and list isn't empty all are online
    return (len(controllers) > 0)

def IsCompOnline(sysComp):
    '''
    check if the controller of a component is online
    '''
    global controllers
    for c in controllers:
        if isinstance(c, sysComp.controllerClass):
            return c.online
        
    # if reached here, it's not online
    return False

def GetController(controllerClass):
    '''
    return the active controller instance of this class
    '''
    global controllers
    for c in controllers:
        if isinstance(c, controllerClass):
            return c
        
    # if reached here, no controller exists
    return None

def ResourcePath(relativePath=''):
    '''
    Get the resource path
    '''
    
#         """ Get absolute path to resource, works for dev and for PyInstaller """
#         try:
#             # PyInstaller creates a temp folder and stores path in _MEIPASS
#             basePath = sys._MEIPASS
#         except Exception:
#             basePath = os.path.dirname(resource_filename('instrumentino.resources', 'main.xrc'))
    basePath = os.path.dirname(resource_filename('instrumentino.resources', 'main.xrc'))
    return os.path.join(basePath, relativePath)


def GetOrCreateDirectory(name):
    '''
    Enter a directory and create it if needed
    '''
    path = name + '/'
        
    try:
        os.mkdir(path)
    except:
        pass
    
    return path

def UserFilesPath(relativePath=''):
    '''
    Get the user directory
    '''
    return GetOrCreateDirectory('user') + relativePath

def LogPath(relativePath=''):
    '''
    Get the log directory
    '''
    return GetOrCreateDirectory('log') + relativePath
    
def Log(text):
    '''
    Log an event
    '''
    global logTextCtrl
    global commandsLogFile
    
    if logTextCtrl != None:
        logTextCtrl.WriteText(text + '\r')
    if commandsLogFile != None:
        commandsLogFile.write(text + '\r')
        
def LogFromOtherThread(text, critical=False):
    '''
    Log an event while running a method/sequence
    '''
    global mainFrame
    wx.PostEvent(mainFrame, ResultEvent(EVT_LOG_UPDATE, (text, critical)))

def UpdateControlsFromOtherThread(runningOperation=False):
    '''
    Update the control buttons while running a method/sequence
    '''
    global mainFrame
    wx.PostEvent(mainFrame, ResultEvent(EVT_UPDATE_CONTROLS, runningOperation))

def Sleep(seconds):
    '''
    Sleep, and wake up when user pressed the stop button
    '''
    time.sleep(seconds%1)
    end = time.time() + int(seconds)
    while time.time() <= end: 
        if userStopped:
            return
        
class ResultEvent(wx.PyEvent):
    '''
    Simple event to carry arbitrary result data.
    '''
    def __init__(self, eventType, data):
        wx.PyEvent.__init__(self)
        self.SetEventType(eventType)
        self.data = data


from __future__ import division
__author__ = 'yoelk'

import wx
from wx import xrc
import copy
import sys
from instrumentino import cfg
import pickle
import wx.lib.masked as  masked
from executable_listctrl import RunnableItem, ExecutableListCtrl

class SysMethod(RunnableItem):
    '''
    Descriptor for a method file to be Run
    '''
    def __init__(self, methodFileName='', repeat=1):
        self.methodFileName = methodFileName
        self.repeat = repeat

    def setRepeatPanel(self, parent):
        '''
        Set and return the repeat panel 
        '''
        sizer = wx.BoxSizer(wx.HORIZONTAL)
        panel = wx.Panel(parent)
        panel.SetSizer(sizer)
        self.maskedTextCtrl = masked.NumCtrl(panel,
                                             value=self.repeat,
                                             integerWidth=cfg.numIntegerPartWidth,
                                             fractionWidth=0,
                                             min=0)
        self.maskedTextCtrl.Bind(wx.EVT_KEY_UP, self.onKeyPress)
        sizer.Add(self.maskedTextCtrl, 0, wx.EXPAND)
        return panel
        
    def onKeyPress(self, event):
        '''
        Update the repeat value when changed in panel
        '''
        self.repeat = self.maskedTextCtrl.GetValue()
        event.Skip()

    def checkSystemCompatibility(self, filename):
        '''
        Check if the file was created by this system
        '''
        if filename != '':
            try:
                fp = file(filename, 'r')
                savedFile = pickle.load(fp)
                if savedFile.systemUid != cfg.systemUid:
                    filename = ''
                    errDlg = wx.MessageDialog(cfg.mainFrame, 'File incompatible with this system',
                                              'Error',
                                              wx.OK | wx.ICON_ERROR)
                    errDlg.ShowModal()
                    errDlg.Destroy()
                    return False
                    
            except (IOError, EOFError):
                errDlg = wx.MessageDialog(cfg.mainFrame, 'Error loading the file',
                                          'Error',
                                          wx.OK | wx.ICON_ERROR)
                errDlg.ShowModal()
                errDlg.Destroy()
                return False
                
            fp.close()
            return True
        else:    
            return True

    def onPathChanged(self, evt):
        '''
        Update the file name when changed in panel
        '''
        filename = evt.GetString()
        if self.checkSystemCompatibility(filename):
            self.methodFileName = filename
        else:
            evt.GetEventObject().SetValue('')
                 
    def Run(self):
        '''
        Run the method
        '''        
        fp = file(self.methodFileName, 'r')
        savedFile = pickle.load(fp)
        actions = savedFile.list
        cfg.LogFromOtherThread('--- ' + self.methodFileName + ' ---')
        for idx in range(self.repeat):
            if self.repeat > 1:
                cfg.LogFromOtherThread('Run no. ' + str(idx + 1))
            for action in actions:
                action.Run()
                if cfg.userStopped:
                    return
                
    def __getstate__(self):
        '''
        Copy the object's state from self.__dict__ which contains
        all our instance attributes. Always use the dict.copy()
        method to avoid modifying the original state.
        '''
        state = self.__dict__.copy()
        # Remove the unpicklable entries.
        del state['maskedTextCtrl']
        return state


class ActionsListCtrl(ExecutableListCtrl):
    '''
    A list of actions to be Run
    '''
    def __init__(self, parent, availableActions, actions=[]):
        self.availableActions = availableActions
        self.availableActionsDict = {action.name: action for action in self.availableActions}
        ExecutableListCtrl.__init__(self, parent, xrc.XRCCTRL(parent, 'actionsListPanel'), {1: 'Action', 2: 'Parameters'}, '--- Run method ---', actions) 
   
    def getDefaultDataItem(self):
        '''
        Return an empty action item
        '''
        return copy.deepcopy(self.availableActions[0]) if len(self.availableActions) > 0 else None 
     
    def getFirstColumnWidget(self, panel, listDataItem):
        '''
        Return the widget for the first column (a choice in this case)
        '''
        choice = wx.Choice(panel, -1, choices=[availableAction.name for availableAction in self.availableActions])
        choice.SetStringSelection(listDataItem.name)
        choice.Bind(wx.EVT_CHOICE, self.onActionChange)
        return choice 
     
    def setOtherColumns(self, index, listDataItem):
        '''
        Set the other columns' content
        '''
        item = self.list.GetItem(index, self.columnNameToNum['Parameters'])
        item.SetWindow(listDataItem.SetParamsPanel(self.list), expand=True)
        self.list.SetItem(item)
     
    def onActionChange(self, event):
        '''
        Change the action when changed in the panel
        '''
        action = copy.deepcopy(self.availableActionsDict[event.GetString()])
        index = int(event.GetEventObject().GetParent().GetName()) - 1
        self.list.GetItemData(index).pop()
        self.list.GetItemData(index).append(action)
        self.setOtherColumns(index, action)
        self.listUpdate()
